-- ***************************************************************************
-- ***************************************************************************
-- ***************************************************************************
-- ***************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library unisim;
use unisim.all;

entity util_spi_3w is
  port (
    m_clk             : in  std_logic;
    x_csn             : in  std_logic_vector(1 downto 0);
    x_clk             : in  std_logic;
    x_mosi            : in  std_logic;
    x_miso            : out std_logic;
    spi_cs0n          : out std_logic;
    spi_cs1n          : out std_logic;
    spi_clk           : out std_logic;
    spi_sdio_T        : out std_logic;
    spi_sdio_O        : out std_logic;
    spi_sdio_I        : in  std_logic;

    debug_trigger     : out std_logic_vector(7 downto 0);
    debug_data        : out std_logic_vector(63 downto 0)
  );
end util_spi_3w;

architecture IMP of util_spi_3w is

  signal  x_clk_d     : std_logic := '0';
  signal  x_csn_d     : std_logic := '0';
  signal  m_enable    : std_logic := '0';
  signal  m_rdwr      : std_logic := '0';
  signal  m_bitcnt    : std_logic_vector(5 downto 0) := (others => '0');
  signal  m_clkcnt    : std_logic_vector(5 downto 0) := (others => '0');

  signal  x_csn_s     : std_logic;

begin

  x_csn_s <= not(x_csn(0) and x_csn(1));
  x_miso <= spi_sdio_I;
  spi_cs0n <= x_csn(0);
  spi_cs1n <= x_csn(1);
  spi_clk <= x_clk;
  spi_sdio_T <= x_csn_s and m_enable;
  spi_sdio_O <= x_mosi;

  debug_trigger <= "0000000" & x_csn_s;
  debug_data(63 downto 23) <= (others => '0');
  debug_data(22) <= m_rdwr;
  debug_data(21) <= x_csn(1);
  debug_data(20) <= x_csn(0);
  debug_data(19) <= x_clk;
  debug_data(18) <= x_mosi;
  debug_data(17) <= x_clk_d; 
  debug_data(16) <= x_csn_d; 
  debug_data(15) <= x_csn_s; 
  debug_data(14) <= x_csn_s and m_enable;
  debug_data(13) <= spi_sdio_I;
  debug_data(12) <= m_enable;
  debug_data(11 downto  6) <= m_bitcnt;
  debug_data( 5 downto  0) <= m_clkcnt;

  p_cnts: process(m_clk) begin
    if (m_clk'event and m_clk = '1') then
      x_clk_d <= x_clk;
      x_csn_d <= x_csn_s;
      if ((m_bitcnt = 16) and (m_clkcnt = 10)) then
        m_enable <= m_rdwr;
      elsif ((x_csn_s = '0') and (x_csn_d = '1')) then
        m_enable <= '0';
      end if;
      if ((x_csn_s = '1') and (x_csn_d = '0')) then
        m_rdwr <= '0';
        m_bitcnt <= (others => '0');
      elsif ((x_clk = '1') and (x_clk_d = '0')) then
        if (m_bitcnt = 0) then
          m_rdwr <= x_mosi;
        end if;
        m_bitcnt <= m_bitcnt + 1;
      end if;
      if ((x_clk = '1') and (x_clk_d = '0')) then
        m_clkcnt <= (others => '0');
      else
        m_clkcnt <= m_clkcnt + 1;
      end if;
    end if;
  end process;

end IMP;

-- ***************************************************************************
-- ***************************************************************************
